# copyright (C) 1997-2000 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: prntview.tcl,v 1.2.1.3 2000/01/23 21:57:11 jfontain Exp $}


class printViewer {

    set ::env(DISPLAY) [winfo screen .]                                                         ;# to make sure gs outputs to screen
    set printViewer::(pixelsPerInch) [winfo pixels . 1i]
    set printViewer::(pixelsPerMillimeter) [expr {$printViewer::(pixelsPerInch)/25.4}]
    set printViewer::(margin) 10
    set printViewer::(offset) 3
    set printViewer::(pageHeight) 130                                                               ;# initial values in millimeters
    set printViewer::(pageWidth) 100

    proc printViewer {this parentPath args} composite {[new frame $parentPath -background gray] $args} {
        set path $widget::($this,path)
        set printViewer::($this,shadow) [frame $path.shadow -background black]
        set printViewer::($this,sheet) [frame $path.sheet -container 1 -background white]
        scan [winfo id $printViewer::($this,sheet)] 0x%x printViewer::($this,id) 
        set printViewer::($this,height) [expr {round($printViewer::(pageHeight)*$printViewer::(pixelsPerMillimeter))}]  ;# in pixels
        set printViewer::($this,width) [expr {round($printViewer::(pageWidth)*$printViewer::(pixelsPerMillimeter))}]
        composite::complete $this
    }

    proc ~printViewer {this} {
        terminateProcess $this
        if {$composite::($this,-deletefile)} {
            file delete $composite::($this,-file)
        }
    }

    proc options {this} {
        return [list\
            [list -deletefile deleteFile DeleteFile 0 0]\
            [list -file file File {} {}]\
            [list -pageheight pageHeight PageHeight $printViewer::(pageHeight)]\
            [list -pagewidth pageWidth PageWidth $printViewer::(pageWidth)]\
            [list -zoom zoom Zoom 1 1]\
        ]
    }

    proc set-deletefile {this value} {}                                                   ;# delete source postscript file when done

    proc set-file {this value} {}

    proc set-pageheight {this value} {                                                                             ;# in millimeters
        set printViewer::($this,height) [expr {round($value*$printViewer::(pixelsPerMillimeter))}]                      ;# in pixels
        displaySheet $this
    }
    proc set-pagewidth {this value} {
        set printViewer::($this,width) [expr {round($value*$printViewer::(pixelsPerMillimeter))}]
        displaySheet $this
    }

    proc set-zoom {this value} {}

    proc terminateProcess {this} {                                                   ;# eventually terminate existing viewer process
        catch {puts $printViewer::($this,channel) quit}
        catch {close $printViewer::($this,channel)}
    }

    proc displaySheet {this} {
        set ratio $composite::($this,-zoom)
        set width [expr {round($printViewer::($this,width)*$ratio)}]
        set height [expr {round($printViewer::($this,height)*$ratio)}]
        place $printViewer::($this,sheet) -x $printViewer::(margin) -y $printViewer::(margin) -width $width -height $height
        place $printViewer::($this,shadow) -x [expr {$printViewer::(margin)+$printViewer::(offset)}]\
            -y [expr {$printViewer::(margin)+$printViewer::(offset)}] -width $width -height $height
        $widget::($this,path) configure -width [expr {$width+(2*$printViewer::(margin))+$printViewer::(offset)}]\
            -height [expr {$height+(2*$printViewer::(margin))+$printViewer::(offset)}]
    }

    proc refresh {this} {
        if {[string length $composite::($this,-file)]==0} return
        terminateProcess $this
        displaySheet $this
        set ratio $composite::($this,-zoom)
        set width [expr {round($printViewer::($this,width)*$ratio)}]
        set height [expr {round($printViewer::($this,height)*$ratio)}]
        set pixelsPerInch [expr {round($printViewer::(pixelsPerInch)*$ratio)}]
        # use open instead of exec so that application is not hung while previewing
        set printViewer::($this,channel) [open \
"|gs -q -dWindowID=$printViewer::($this,id) -g${width}x$height -r$pixelsPerInch -dBATCH -dNOPROMPT $composite::($this,-file)" w\
        ]
        fconfigure $printViewer::($this,channel) -blocking 0                                       ;# required to prevent hang state
    }

}
